"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var os_1 = __importDefault(require("os"));
var path_1 = __importDefault(require("path"));
var getopts_1 = __importDefault(require("getopts"));
var node_sass_1 = __importDefault(require("node-sass"));
var glob_1 = __importDefault(require("glob"));
// import chokidar from 'chokidar';
var postcss_1 = require("postcss");
var postcss_selector_parser_1 = __importDefault(require("postcss-selector-parser"));
var options = getopts_1.default(process.argv.slice(2), {
    alias: {
        watch: 'w',
        pattern: 'p',
        ignore: 'i',
        outputDir: 'o',
    },
    default: {
        watch: false,
        pattern: '**/*.{scss,css}',
        ignore: '**/node_modules/**',
        outputDir: '',
    },
    boolean: ['watch'],
    string: ['pattern', 'ignore', 'outputDir'],
});
// TODO...
// if (options.watch) {
//  const watcher = chokidar.watch...
// }
var pattern = options.pattern;
var ignore = options.ignore;
var inputDir = options._[0] || '.';
var outputDir = options.outputDir || inputDir;
var searchFiles = function (pattern, inputDir, ignore) {
    var options = {
        cwd: path_1.default.resolve(process.cwd(), inputDir),
        ignore: ignore,
    };
    return new Promise(function (resolve, reject) {
        return glob_1.default(pattern, options, function (error, files) { return (error ? reject(error) : resolve(files)); });
    });
};
var readFile = function (file) {
    return new Promise(function (resolve, reject) {
        node_sass_1.default.render({ file: file, includePaths: ['node_modules/'] }, function (error, result) {
            if (error) {
                reject(error);
            }
            else {
                resolve(result.css.toString());
            }
        });
    });
};
var writeFile = function (file, content) {
    return new Promise(function (resolve, reject) {
        fs_1.default.mkdir(path_1.default.dirname(file), { recursive: true }, function (error) {
            if (error) {
                reject(error);
            }
            else {
                fs_1.default.writeFile(file, content, function (error) {
                    if (error) {
                        reject(error);
                    }
                    else {
                        resolve(file);
                    }
                });
            }
        });
    });
};
var createTypes = function (classList) {
    if (classList.length === 0) {
        return '';
    }
    return __spreadArrays([
        'declare const styles: {'
    ], classList.map(function (selector) { return "  readonly " + selector + ": string;"; }), [
        '};',
        'export = styles;',
        '',
    ]).join(os_1.default.EOL);
};
var parseFile = function (content) {
    var classList = new Set();
    var camelCaseDashesOnly = function (value) {
        return value.replace(/-+(\w)/g, function (_, char) { return char.toUpperCase(); });
    };
    postcss_1.parse(content).walkRules(function (rule) {
        postcss_selector_parser_1.default()
            .astSync(rule.selector)
            .walkClasses(function (classNode) {
            classList.add(camelCaseDashesOnly(classNode.value));
        });
    });
    return Array.from(classList);
};
searchFiles(pattern, inputDir, ignore)
    .then(function (fileList) {
    var count = 0;
    return Promise.all(fileList.map(function (filename) {
        var readPath = path_1.default.resolve(inputDir, filename);
        var writePath = path_1.default.resolve(outputDir, filename + ".d.ts");
        return readFile(readPath)
            .then(function (fileContent) {
            return parseFile(fileContent);
        })
            .then(function (classList) {
            return createTypes(classList);
        })
            .then(function (typeContent) {
            return writeFile(writePath, typeContent);
        })
            .then(function () {
            count++;
        })
            .catch(function (error) {
            console.error(error.message);
        });
    }))
        .then(function () {
        return count;
    })
        .catch(function (error) {
        return Promise.reject(error);
    });
})
    .then(function (count) {
    var fileCount = count === 1 ? count + " file" : count + " files";
    console.log("Creating typings for " + fileCount);
})
    .catch(function (error) {
    console.error(error.message);
});
